/* RBC_SecondOrder.prg  */

// Solves the RBC model using Schmitt-Grohe / Uribe Second-Order Perturbation Method

new;
cls;
dlibrary -a PszTgSen,PsdTgSen;
#include gensys.src

/* establish global variables */

neulers = 1;                        // # of euler equations included in the model
nexstates = 1;                      // # of structural shocks
nendstates = 1;                     // # of endogenous state variables
ncontrols = 5;                      // # of control variables 
nstates = nendstates+nexstates;     // total # of state variables
nvars = ncontrols+nstates;          // # of variables included in the model
xbar = 0;                           // will contain ss values
xstar = 0;                          // ss values extended 
approx = 2;                         // Order of approximation desired
procvec=0;                          // Define a system of equations as a vector of procedures


// establish parameters: alpha, beta, delta, rho, sigeps, phi, psi
let p[7,1] = 0.24 0.99 0.025 0.78, 0.0067, 1.55, 0.35;

eta = zeros(nstates,nexstates) ; //Matrix defining driving force
sigma = 1;		// perturbation parameter
vcvmat = zeros(nexstates,nexstates);	// VCV matrix of exogenous innovations
vcvmat[1,1] = p[5]^2;
sqrtvcvmat = chol(vcvmat)';
eta[nendstates+1:nstates,.] = sqrtvcvmat;


/* load procedures needed to solve model */
#include RBC_SecondOrder.src
#include schmitt.src

ss = steady(p);

// Controls first, then endogenous states, then exogenous states
// order: y,c,i,n,l,k,a

// Obtain quadratic approximation of Schmitt-Grohe:

{fx,fxp,fy,fyp,fypyp,fypy,fypxp,fypx,fyyp,fyy,fyxp,fyx,fxpyp,fxpy,fxpxp,fxpx,fxyp,fxy,fxxp,fxx,gx,hx}=first_order(p,ncontrols,nstates);
{gxx,hxx} = gxx_hxx(fx,fxp,fy,fyp,fypyp,fypy,fypxp,fypx,fyyp,fyy,fyxp,fyx,fxpyp,fxpy,fxpxp,fxpx,fxyp,fxy,fxxp,fxx,hx,gx) ;
{gss,hss} = gss_hss(fx,fxp,fy,fyp,fypyp,fypy,fypxp,fypx,fyyp,fyy,fyxp,fyx,fxpyp,fxpy,fxpxp,fxpx,fxyp,fxy,fxxp,fxx,hx,gx,gxx,eta);



// Graph policy functions

lb = -0.1;
up = 0.1;
range = up-lb;
nsteps = 101;
step = range/(nsteps-1);
tildevals = seqa(lb,step,nsteps);

cees = zeros(nsteps,4);
ks = zeros(nsteps,1);
as = zeros(nsteps,1);
controlind = 3; // 1=y, 2=c, 3=i, 4=n, 5=l

iii=1; do while iii<=nsteps;
    stilde = tildevals[iii];
    ks[iii] = ss[ncontrols+1].*exp(stilde);
    as[iii] = ss[ncontrols+nendstates+1].*exp(stilde);
    csp = csp_quad_of_s(ks[iii]|ss[ncontrols+nendstates+1],sigma);
    cees[iii,1] = csp[controlind];
    csp = csp_lin_of_s(ks[iii]|ss[ncontrols+nendstates+1]);
    cees[iii,2] = csp[controlind];
    csp = csp_quad_of_s(ss[ncontrols+1]|as[iii],sigma);
    cees[iii,3] = csp[controlind];
    csp = csp_lin_of_s(ss[ncontrols+1]|as[iii]);
    cees[iii,4] = csp[controlind];
iii=iii+1; endo;

cees = ln(cees./(ss[controlind]));
ks = ln(ks./(ss[ncontrols+1]));
as = ln(as./(ss[ncontrols+nendstates+1]));

slopes = cees[2:nsteps,.]-cees[1:nsteps-1,.];

library pgraph;

_pdate="";
begwind;
window(2,2,0);
xy(ks,cees[.,1 2]);
nextwind;
xy(as,cees[.,3 4]);
nextwind;
xy(ks[2:nsteps],slopes[.,1 2]);
nextwind;
xy(as[2:nsteps],slopes[.,3 4]);
endwind;

// Simulate data, calculate moments


// prepare for mc integration: draw epsd epsq pairs

rndseed 987654321;
ndraws = 100000;
draws = zeros(ndraws,nstates); 
iii=1; do while iii<=ndraws; 
    draws[iii,.] = (eta*sigma*rndn(nexstates,1)')'; 
iii=iii+1; endo;

// simulate data, calculate summary stats

    artdat = zeros(ndraws+1,nvars*2); // first set of columns: linear; second set, non-linear
    artdat[1,.] = ss'~ss';
    jjj = 1; do while jjj<=ndraws;
        // quadratic
        csp = csp_quad_of_s(artdat[jjj,ncontrols+1:nvars]',sigma);
        artdat[jjj,1:ncontrols] = csp[1:ncontrols]';
        artdat[jjj+1,ncontrols+1:nvars] = csp[ncontrols+1:nvars]'.*exp(draws[jjj,.]);
        // linear
        csp = csp_lin_of_s(artdat[jjj,ncontrols+1:nvars]');
        artdat[jjj,nvars+1:nvars+ncontrols] = csp[1:ncontrols]';
        artdat[jjj+1,nvars+ncontrols+1:2*nvars] = csp[ncontrols+1:nvars]'.*exp(draws[jjj,.]);
    jjj=jjj+1; endo;

artdat = artdat[1:ndraws,.];
artdat = ln(artdat./(ss'~ss'));
stds = stdc(artdat);

"standard deviations and relative standard deviations, quadratic versus linear approximation:";?;

stds[1:nvars]~(stds[1:nvars]./stds[1])~stds[nvars+1:2*nvars]~(stds[nvars+1:2*nvars]./stds[nvars+1]);

// construct impulse response functions

nresps = 100;
draws = zeros(nresps,nstates); 
iii=1; do while iii<=ndraws; 
if iii==1;
    draws[iii,.] = (eta*sigma)'; 
endif;
iii=iii+1; endo;

// simulate data

    artdat = zeros(nresps+1,nvars*2); // first set of columns: linear; second set, non-linear
    artdat[1,.] = ss'~ss';
    jjj = 1; do while jjj<=nresps;
        // quadratic
        csp = csp_quad_of_s(artdat[jjj,ncontrols+1:nvars]',sigma);
        artdat[jjj,1:ncontrols] = csp[1:ncontrols]';
        artdat[jjj+1,ncontrols+1:nvars] = csp[ncontrols+1:nvars]'.*exp(draws[jjj,.]);
        // linear
        csp = csp_lin_of_s(artdat[jjj,ncontrols+1:nvars]');
        artdat[jjj,nvars+1:nvars+ncontrols] = csp[1:ncontrols]';
        artdat[jjj+1,nvars+ncontrols+1:2*nvars] = csp[ncontrols+1:nvars]'.*exp(draws[jjj,.]);
    jjj=jjj+1; endo;

artdat = ln(artdat[2:nresps,.]./(ss'~ss'));
horizon = seqa(0,1,nresps-1);

ytics(-0.005,0.015,0.005,0);
begwind;
window(3,3,0);
iii=1; do while iii<=nvars;
    if iii==3;
        ytics(-0.01,0.06,0.01,0);
        xy(horizon,artdat[.,iii iii+nvars]~zeros(nresps-1,1));
        ytics(-0.005,0.015,0.005,0);
        nextwind;
    else;
        xy(horizon,artdat[.,iii iii+nvars]~zeros(nresps-1,1));
        nextwind;
    endif;
iii=iii+1; endo;
endwind;



proc csp_quad_of_s(s,sig);

    // constructs policy function using quadratic approximation
    // inputs are levels of s; outputs are levels of sp, c

    local stilde,s2tilde,conttilde,sptilde,cont,sp,ii;

    stilde = ln(s./ss[ncontrols+1:nvars]);
    s2tilde = stilde*stilde';

    conttilde=zeros(ncontrols,1);
    sptilde=zeros(nstates,1);

    ii=1; do while ii<=nstates;
        conttilde = conttilde + 0.5*getMatrix(gxx[.,.,.],ii)*s2tilde[ii,.]';
        sptilde = sptilde + 0.5*getMatrix(hxx[.,.,.],ii)*s2tilde[ii,.]';
	ii=ii+1;endo;
    
    conttilde = gx*stilde + conttilde + 0.5*gss*sig^2;
    sptilde = hx*stilde + sptilde + 0.5*hss*sig^2;

    cont = ss[1:ncontrols].*exp(conttilde);
    sp = ss[ncontrols+1:nvars].*exp(sptilde);

    retp(cont|sp);
endp;

proc csp_lin_of_s(s);

    // constructs policy function using linear approximation
    // inputs are levels; outputs are levels

    local stilde,conttilde,sptilde,cont,sp;

    stilde = ln(s./ss[ncontrols+1:nvars]);

    conttilde = gx*stilde;
    sptilde = hx*stilde;

    cont = ss[1:ncontrols].*exp(conttilde);
    sp = ss[ncontrols+1:nvars].*exp(sptilde);

    retp(cont|sp);
endp;
